function [FFCFpars, spec, parsnew] = CAFE(CLSpars,fwhm,errprop,varargin)
    
%{
CAFE is a program that returns FFCF parameters from CLS fit parameters and
the FWHM of the associated absorption spectrum. See associated publication
for more details.
https://pubs.acs.org/doi/abs/10.1021/acs.jpca.0c04313

The input parameters are:

    CLSpars is a column vector of the parameterized multiexponential CLS decay with the
    following values. Enter up to 7 parameters.

    CLSpars(1) = A_inf (CLS offset)
    CLSpars(2) = A_1
    CLSpars(3) = tau_1 (ps)
    CLSpars(4) = A_2
    CLSpars(5) = tau_2 (ps)
    CLSpars(6) = A_3
    CLSpars(7) = tau_3 (ps)

    fwhm is the FWHM of the absorption spectrum in (cm^-1)

    errprop is the mode of error propagation
        1: Neural Network Only

        2: Estimate Error from CLS parameter std. error
            varargin{1} = column vector of error parameters of CLS
                          parameters in same order as CLSpars
            varargin{2} = error value for FWHM, if applicable

        3: Calculate Error from CLS parameter covariance matrix. Necessary
           for best results and some error correction algorithms.
            varargin{1} = Covariance matrix for the input CLS parameters,
                          in the same order as the input CLS parameters
            varargin{2} = error value for FWHM, if applicable

The output parameters are:

    FFCFpars are the calculated FFCF parameters

    FFCFpars(1) = Delta_inf (cm^-1)
    FFCFpars(2) = Delta_1   (cm^-1)
    FFCFpars(3) = Delta_1   (cm^-1)
    FFCFpars(4) = Delta_2   (cm^-1)
    FFCFpars(5) = Gamma     (cm^-1)
    FFCFpars(6) = T_2       (ps)

    spec is the calculated 1D absorption spectrum from the calculated FFCF
    First column is a frequency column (cm-1)
    Second column is the normalized spectrum

    parsnew is the set of CLS parameters. May be different from input if
    corrections are applied.
%}
    suppfcndir = 'CAFEfcns'; %Change if folder is not in same directory 
    addpath(suppfcndir)

    disp('Thank you for using CAFE v1.1.');
    disp('If you find this program helpful in your research, remember to <a href="https://pubs.acs.org/doi/abs/10.1021/acs.jpca.0c04313">cite the publication!</a>');
    disp('Calculating FFCF...');

    %Pad CLSpars if less than 7 parameters entered
    if length(CLSpars) < 7
        CLSpars(7,1) = 0;
    end

    %Split CLSpars into amplitudes and time constants
    A = [CLSpars(2) CLSpars(4) CLSpars(6) CLSpars(1) ];
    tau = [CLSpars(3) CLSpars(5) CLSpars(7)];
    
    %Input validation
    flag = 0;
    if sum(any(isnan(CLSpars)))>0           %Check inputs are numbers
        beep;
        flag = 1;
        disp('Please enter numbers for all inputs!');
    end
    if sum(A) > 1                           %Check CLS starts below 1
        beep;
        flag = 1;
        disp('CLS starts above 1!');
    end
    if (sum(abs(imag(CLSpars)))) > 0        %Check inputs are real
        beep;
        flag = 1;
        disp('Input parameters have imaginary components!');
    end
    if any(A < 0)                           %Check all amplitudes are positive
        beep;
        flag = 1;
        disp('CLS has negative components!');
    end
    for i = 1:3                             %Check used time constants are > 0
        if tau(i) <= 0 && A(i) > 0
            beep;
            flag = 1;
            disp('Time Constants of CLS components must be greater than 0!');
        end
    end
    if (fwhm <= 0)                          %Check FWHM is greater than 0
        beep;
        flag = 1;
        disp('FWHM has to be greater than 0!');
    end
    if (errprop == 2)
        if nargin < 4
            beep;
            flag = 1;
            disp('Please enter the CLS parameter error values!');
        end
    end
    if (errprop == 3)
        if nargin < 4
            beep;
            flag = 1;
            disp('Please enter the CLS parameter covariance matrix!');
        end
    end

    if ~flag                                %If no errors, proceed
       
        pars = [CLSpars; fwhm];             %concatenate pars
        
        
        %Find all non-zero CLS components, sort tau from fastest to slowest
        posAind = find(A(1:3) > 0);
        zeroAind = find(A(1:3) <= 0);
        [tau_sort, sorttauind] = sort(tau(posAind));
        sortind = [sorttauind zeroAind];
        
        sortparsind = [1 2*sortind(1) 2*sortind(1)+1 2*sortind(2) 2*sortind(2)+1 2*sortind(3) 2*sortind(3)+1 8];
        
        pars_sort = pars(sortparsind);
        
        tau2 = ones(1,3);
        tau2(1:length(tau_sort)) = tau_sort;
        tau_sort = tau2;
        
        tau2 = ones(1,3);
        tau2(1:length(posAind)) = tau(posAind);
        tau = tau2;

        unsortind = 1:3;
        unsortind = unsortind(posAind);
        unsortind = unsortind(sorttauind);

        if errprop == 3       %If using covariance matrix
            covar = varargin{1};
            if nargin > 4     %If using fwhm error append to end
                covar(8,8) = varargin{2}.^2;
            else
                covar(8,8) = 0;
            end
            covar_sort = covar(sortparsind,sortparsind); %sort parameters
        elseif errprop == 2   %If using std. error approximation
            stderr = varargin{1};
            if nargin > 4     %If using fwhm error append to end
                stderr(8,1) = varargin{2};
            else
                stderr(8,1) = 0;
            end
            %correlation matrix estimation
            corr = [1,0,0,0,0,0,0,0;...
                    0,1,-0.75,0,0,0,0,0; ...
                    0,-0.75,1,0,0,0,0,0;...
                    0,0,0,1,-0.75,0,0,0;...
                    0,0,0,-0.75,1,0,0,0;...
                    0,0,0,0,0,1,-0.75,0;...
                    0,0,0,0,0,-0.75,1,0;...
                    0,0,0,0,0,0,0,1];
            covar = stderr*stderr'.*corr; %calculate covariance estimate
            covar_sort = covar(sortparsind,sortparsind); %sort parameters
        else %if using only NN error
            covar_sort = zeros(8,8);            
        end
        
        try
            [algoout, parsnew, msg] = CLS2FFCFalgo(pars_sort,covar_sort,errprop); %attempt to calculate FFCF parameters
        catch ME1
            beep;
            disp(ME1.message);
            return;
        end
        taunew = [parsnew(3) parsnew(5) parsnew(7)];
        
        [~, omega, S1] = calcFWHMfromFFCF(algoout(1:4), taunew, algoout(5)); %Get 1D Spectrum from FFCF
        spec = [omega',real(S1)];
        disp('Calculation Complete')

        
        Delta = zeros(4,2);
        Delta(unsortind,:) = algoout(1:length(unsortind),:);
        Delta(4,:) = algoout(4,:);
        %"unsort" Delta parameters to match original input
        
        Gamma = algoout(5,:);
        FFCFpars = zeros(6,2);
        FFCFpars(1:5,:) = [Delta([4 1 2 3],:); Gamma];

        %Calculate T2 from Gamma
        T2 = 1/(pi*0.0299792458*algoout(5,1));
        T2_err = T2*algoout(5,2)/algoout(5,1);
        FFCFpars(6,:) = [T2 T2_err];
        
        %Format parameters for display
        FFCFparsFormat = {};
        for i = 1:numel(FFCFpars)
            
            tempStr = sprintf('%8g', round(FFCFpars(i),2+(i<7),'significant'));
            FFCFparsFormat{i} = tempStr;
        end
        FFCFlabels{1} = 'Delta_Inf (cm-1)';
        FFCFlabels{2} = 'Delta_1   (cm-1)';
        FFCFlabels{3} = 'Delta_2   (cm-1)';
        FFCFlabels{4} = 'Delta_3   (cm-1)';
        FFCFlabels{5} = 'Gamma     (cm-1)';
        FFCFlabels{6} = 'T_2       (ps)  ';
        %Display FFCF parameters
        disp(reshape([FFCFlabels FFCFparsFormat],6,3))
        parsnew = parsnew';
        
        if (msg)
            disp(msg)
            beep;
        end
    else %if flags
        disp('Shutting down...')
    end
    
    rmpath(suppfcndir)
end